/**
 *    Copyright (C) 2021 Graham Leggett <minfrin@sharp.fm>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

/*
 * Common utility functions for all tools.
 */

#include "redwax_util.h"

#include <apr_strings.h>
#include <apr_escape.h>

int redwax_certcmp(redwax_certificate_t *cert1, redwax_certificate_t *cert2)
{

	return 0;
}

apr_size_t redwax_strrcspn(const char *s, const char *charset)
{
	apr_size_t last = 0, cur;

	cur = strcspn(s, charset);

	while (s[cur++]) {
		last = cur;
		cur += strcspn(s + cur, charset);
	}

	return last;
}

const char *redwax_stroff(const char *s, apr_size_t off)
{
	while(off-- && *s) {
		s++;
	}

	return s;
}

const char *redwax_pstrntrim(apr_pool_t *p, const char *s, apr_size_t off)
{
	const char *start, *end;
	apr_size_t remain = off;

	while (remain) {

		if (*s == ' ') {
			s++;
			remain--;
		}
		else {
			break;
		}

	}

	start = end = s;

	while (remain) {

		if (*s != ' ') {
			end = s + 1;
		}

		s++;
		remain--;
	}

	return apr_pstrndup(p, start, end - start);
}

const char *redwax_pescape_echo(apr_pool_t *p, const char *str,
        redwax_token_quoted_e quoted, int close)
{

     str = apr_pescape_echo(p, str, (quoted != REDWAX_TOKEN_NOQUOTE));

     if (str) {

         if (quoted != REDWAX_TOKEN_NOQUOTE) {

             char *d;
             const char *s;

             int i;
             int count = 1 + close;

             /* count quotes */
             for (i = 0; str[i]; i++) {
                 if (str[i] == '\"') {
                     count++;
                 }
             }

             /* escape quotes */
             s = str;
             d = apr_palloc(p, strlen(str) + count + 1);
             str = d;

    		 *d++ = '\"';

             for (; *s; ++s) {
            	 if (*s == '\"') {
            		 *d++ = '\\';
            	 }
            	 *d++ = *s;
             }

             *d = 0;

        	 if (close) {
        		 *d++ = '\"';
        	 }

         }
         else {

             int i;
             int count = 0;

             /* count spaces */
             for (i = 0; str[i]; i++) {
            	 switch (str[i]) {
            	 case ' ':
            	 case ';':
                     count++;
            		 break;
            	 default:
            		 break;
            	 }
             }

             /* escape spaces */
             if (count) {

                 char *d;
                 const char *s;

                 s = str;
                 d = apr_palloc(p, strlen(str) + count + 1);
                 str = d;

                 for (; *s; ++s) {
                	 switch (*s) {
                	 case ' ':
                	 case ';':
                         *d++ = '\\';
                		 break;
                	 default:
                		 break;
                	 }
                     *d++ = *s;
                 }

                 *d = 0;

             }

         }
     }

     return str;
}

